######################################################################
# Copyright (c) Adrien Luxey-Bitri, Boris Baldassari
#
# This program and the accompanying materials are made
# available under the terms of the Eclipse Public License 2.0
# which is available at https://www.eclipse.org/legal/epl-2.0/
#
# SPDX-License-Identifier: EPL-2.0
######################################################################

"""
A package for learning network programming in Python.

This module (file) manages operations relative to the file system.
"""

import os.path as path


def resolve_location(res:str, root: str):
    """Returns the path of a resource relative to the root and its extension.

    Returns ("", "") if the concatenated path does not exist.

    "index.html" is appended to directory paths.

    Parameters
    ----------
    res: str 
        The queried resource path.
    root: str 
        The root directory where to look into for res.

    Returns
    -------
    str
        The full disk path of the resource if it exists, or "".
    str
        The extension of the resource if it exists, or "".
    """
    try:
        res_path = resolve_path(res, root)

        # Compute file extension
        extension = path.splitext(res_path)[1]
        # Maybe remove leading '.'
        if len(extension) > 0 and extension[0] == '.':
            extension = extension[1:]

        return res_path, extension
    except Exception: 
        return "", ""

def resolve_path(res:str, root: str):
    """Returns the full disk path of a resource relative to the root.

    Returns "" if the concatenated path does not exist.

    "index.html" is appended to directory paths.

    Parameters
    ----------
    res: str 
        The queried resource path.
    root: str 
        The root directory where to look into for res.

    Returns
    -------
    str
        The full disk path of the resource if it exists, or "".
    """
    try:
        # Resolve the home directory if it is in the root
        root = path.expanduser(root)

        if res[-1] == "/":
            res += "index.html"
        while res[0] == "/":
            res = res[1:]

        res_path = path.join(root, res)
        
        if not (path.exists(res_path) and path.isfile(res_path)):
            return ""
        return res_path
    except Exception: 
        return ""



def get_resource(res_path: str):
    """Returns a resource at res_path, its content type and an HTTP code.

    Parameters
    ----------
    - res_path: str
        Requested resource string.

    Returns
    -------
    bytes
        The resource content if it exists (code == 200).
    int 
        A HTTP status code.
    """


    print(f"Opening file {res_path}...")
    with open(res_path, 'rb') as f:
        content = f.read()
        return content, 200

